@@ -1,30 +1,32 @@
-from aquests.protocols.dns import asyndns
+from ..protocols.dns import asyndns
 import time
 
 class DNSCache:
-	def __init__ (self, logger = None):		
+	def __init__ (self, logger = None, dns_servers = []):		
 		self.dns = asyndns.Request (logger)
 		self.logger = logger
+		self.dns_servers = dns_servers
 		self.cache = {}
 		self.hits = 0
 
-	def set (self, answers):
-		if answers:		
-			name = answers [0] ["name"]
-			answer = answers [-1]			
-			for each in answers:
-				name = each ["name"]
-				if name not in self.cache:
-					self.cache [name] = {}			
-				if "ttl" in answer:
-					answer ["valid"]	= time.time () + answer ["ttl"]
-				self.cache [name][answer ["typename"]] = [answer]
-			
+	def set (self, answers):		
+		if not answers:
+			return
+
+		for answer in answers:
+			name = answer ["name"].lower ()
+			if name not in self.cache:
+				self.cache [name] = {}
+			if "ttl" in answer:
+				answer ["valid"]	= time.time () + answer ["ttl"]
+			self.cache [name][answer ["typename"]] = [answer]
+		
 	def expire (self, host):
 		try: del self.cache [host]
 		except KeyError: pass		
 
-	def get (self, host, qtype = "A", check_ttl = False):
+	def get (self, host, qtype = "A", check_ttl = True):	
+		host = host.lower ()
 		try: answers = self.cache [host][qtype]
 		except KeyError: return []
 		answer = answers [0]
@@ -34,7 +36,7 @@ class DNSCache:
 			now = time.time ()
 			if check_ttl and answer ["valid"] < now:				
 				# use max 5 minutes seconds for other querees
-				answer ['valid'] = mow + 300 
+				answer ['valid'] = now + 300
 				# nut new query will be started
 				return []
 			else:
@@ -51,7 +53,7 @@ class DNSCache:
 		
 	def __call__ (self, host, qtype, callback):
 		self.hits += 1
-		hit = self.get (host, qtype, True)
+		hit = self.get (host, qtype, True)		
 		if hit: 
 			return callback (hit)
 	
@@ -60,20 +62,24 @@ class DNSCache:
 			return callback ([{"name": host, "data": host, "typename": qtype}])
 		
 		try:
-			self.dns.req (host, qtype = qtype, protocol = "tcp", callback = [self.set, callback])
+			self.dns.req (host, server = self.dns_servers, qtype = qtype, protocol = "tcp", callback = [self.set, callback])
 		except:
 			self.logger.trace (host)
-			callback ([])
+			hit = [{"name": host, "data": None, "typename": qtype, 'ttl': 60}]
+			self.set (hit)
+			callback (hit)
 
 
+PUBLIC_DNS_SERVERS = ['8.8.8.8', '8.8.4.4']
 query = None
 
-def init (logger):
+def init (logger, dns_servers = []):
+	if not dns_servers:
+		dns_servers = PUBLIC_DNS_SERVERS
 	global query
 	if query is None:
-		query = DNSCache (logger)	
+		query = DNSCache (logger, dns_servers)	
 
 def get (name, qtype = "A"):	
 	global query
 	return query.get (name, qtype, False)
-	
