@@ -6,16 +6,18 @@ import time
 import zlib
 from warnings import warn
 from errno import ECONNRESET, ENOTCONN, ESHUTDOWN, ECONNABORTED, EWOULDBLOCK
+if os.name == "nt":
+	from errno import WSAENOTCONN
 import select
 import threading
 from . import adns
-from aquests.protocols.http2 import H2_PROTOCOLS
+from ..protocols.http2 import H2_PROTOCOLS
+from ..lib.athreads.fifo import await_fifo
+from ..lib.ssl_ import resolve_cert_reqs, resolve_ssl_version, create_urllib3_context
+from collections import deque
+from ..protocols.http import respcodes
 
-from aquests.lib.ssl_ import resolve_cert_reqs, resolve_ssl_version, create_urllib3_context
-
-NPN_PROTOCOL = 'h2'
-H2_NPN_PROTOCOLS = [NPN_PROTOCOL, 'h2-16', 'h2-15', 'h2-14']
-DEBUG = False
+DEBUG = 0
 	
 class SocketPanic (Exception): pass
 class TimeOut (Exception): pass
@@ -24,15 +26,11 @@ class AsynConnect (asynchat.async_chat):
 	ac_in_buffer_size = 65535
 	ac_out_buffer_size = 65535
 	zombie_timeout = 10
-	keep_alive_timeout = 10
-	network_delay_timeout = 10	
-	ready = None
-	affluent = None	
-	
+	keep_alive = 10
 	request_count = 0
 	active = 0
-	proxy = False
-
+	fifo_class = deque
+	
 	def __init__ (self, address, lock = None, logger = None):
 		self.address = address
 		self.lock = lock
@@ -41,38 +39,83 @@ class AsynConnect (asynchat.async_chat):
 		self.__sendlock = None
 		self.__no_more_request = False
 		self.set_event_time ()
-		self.proxy = False
-		self.proxy_client = False
 		self.handler = None
+		
+		self.auth = None
+		self.proxy = False
+		
 		self.initialize_connection ()
-		asynchat.async_chat.__init__ (self)
+		self._closed = False
+		
+		self.ac_in_buffer = b''
+		self.incoming = []
+		self.producer_fifo = self.fifo_class ()
+		asyncore.dispatcher.__init__(self)
 	
+	def duplicate (self):
+		new_asyncon = self.__class__ (self.address, self.lock, self.logger)
+		# used in socketpool
+		new_asyncon.proxy = self.proxy
+		# used in skitai cluster manager
+		new_asyncon.keep_alive = self.keep_alive
+		new_asyncon.auth = self.auth
+		return new_asyncon
+			
+	def __repr__ (self):
+		return "<AsynConnect %s:%d>" % self.address
+	
+	def set_auth (self, auth):
+		self.auth = auth
+	
+	def get_auth (self):
+		return self.auth
+				
 	def close (self):
-		#if self._closed:
-		#	return
+		if self._closed:
+			return
+		
 		if self.socket:
 			# self.socket is still None, when DNS not found
 			asynchat.async_chat.close (self)
-		
+			self._fileno = None
+			
 		# re-init asychat
 		self.ac_in_buffer = b''
 		self.incoming = []
 		self.producer_fifo.clear()
-		
-		if self.handler and self.errcode:		
-			self.handler.connection_closed (self.errcode, self.errmsg)
-		
-		if not self.proxy_client:
-			self.logger ("[info] .....socket %s:%d has been closed" % self.address)
-
-		self.set_active (False)
-		self.handler = None
+		self._proto = None
 		self._closed = True
 			
+		if not self.handler:			
+			# return to the pool
+			return self.set_active (False)		
+		if not self.errcode:
+			# disconnect intentionally
+			return
+		
+		handler, self.handller = self.handler, None
+		keep_active = False			
+		try: 
+			keep_active = handler.connection_closed (self.errcode, self.errmsg)
+		except: 
+			self.trace ()
+		
+		if not keep_active:
+			self.set_active (False)	
+			if self.errcode not in (704, 712):
+				# controlled shutdown
+				self.logger (
+					".....socket %s has been closed (reason: %d)" % ("%s:%d" % self.address, self.errcode),
+					"info"
+				)
+		# DO NOT Change any props, because may be request has been restarted	
+	
 	def end_tran (self):
+		if DEBUG:
+			self.logger ('end_tran {rid:%s} %s' % (self.handler.request.meta.get ('req_id', -1), self.handler.request.uri), 'debug')
 		self.del_channel ()
 		self.handler = None
-		self.set_active (False)
+		self.set_active (False)		
 				
 	def use_sendlock (self):
 		self.__sendlock = threading.Lock ()
@@ -95,7 +138,6 @@ class AsynConnect (asynchat.async_chat):
 		return self.__history
 				
 	def initialize_connection (self):		
-		self._closed = False
 		self._raised_ENOTCONN = 0 # for win32
 		self.__history = []
 		self._proto = None
@@ -104,8 +146,6 @@ class AsynConnect (asynchat.async_chat):
 		
 		self.established = False		
 		self.upgraded = False		
-		self.ready = None
-		self.affluent = None
 		
 	def set_event_time (self):
 		self.event_time = time.time ()
@@ -128,31 +168,19 @@ class AsynConnect (asynchat.async_chat):
 			self.logger.trace ()
 		else:
 			warn ("No logger for traceback")
-				
-	def duplicate (self):
-		return self.__class__ (self.address, self.lock, self.logger)
 		
 	def clean_shutdown_control (self, phase, time_in_this_phase):	
 		self.__no_more_request = True
 		if self.isactive () or (self.handler and self.handler.working ()):
 			return 1
 		else:				
-			self.handle_close ()
+			self.handle_close (712, "Controlled Shutdown")
+			self.__no_more_request = False
 			return 0
-		
-	def readable (self):
-		if self.affluent is not None:
-			return asynchat.async_chat.readable (self)	and self.affluent ()
-		return asynchat.async_chat.readable (self)
 	
-	def writable (self):
-		if self.ready is not None:
-			return asynchat.async_chat.writable (self) and self.ready ()
-		return asynchat.async_chat.writable (self)	
-  
 	def is_channel_in_map (self, map = None):
 		if map is None:
-			map = self._map
+			map = self._map		
 		return self._fileno in map
 		
 	def set_active (self, flag, nolock = False):
@@ -163,11 +191,13 @@ class AsynConnect (asynchat.async_chat):
 			
 		if nolock or self.lock is None:
 			self.active = flag
+			if not flag: self.set_timeout (self.keep_alive)
 			return
 			
 		self.lock.acquire ()
 		self.active = flag
-		self.request_count += 1
+		self.request_count += 1		
+		if not flag: self.set_timeout (self.keep_alive)
 		self.lock.release ()
 	
 	def get_active (self, nolock = False):
@@ -188,76 +218,73 @@ class AsynConnect (asynchat.async_chat):
 		
 	def get_request_count (self):	
 		return self.request_count
-	
-	def add_channel (self, map = None):
-		self.zombie_timeout =  self.network_delay_timeout
-		self._fileno = self.socket.fileno ()
-		return asynchat.async_chat.add_channel (self, map)		
 		
 	def del_channel (self, map = None):
-		asynchat.async_chat.del_channel (self, map)
-		# make default and sometimes reset server'stimeout	
-		self.zombie_timeout =  self.keep_alive_timeout
-				
-	def create_socket (self, family, type):
+		fd = self._fileno
+		if map is None:
+			map = self._map
+		if fd in map:
+			del map[fd]
+            			
+	def create_socket (self, family, type):		
 		self.family_and_type = family, type
 		sock = socket.socket (family, type)
 		sock.setblocking (0)
 		self.set_socket (sock)
-	
+			
 	def connect (self):
+		if DEBUG:
+			self.logger ('query DNS {rid:%s} %s' % (self.handler.request.meta.get ('req_id', -1), self.handler.request.uri), 'debug')
 		if adns.query:
 			adns.query (self.address [0], "A", callback = self.continue_connect)
 		else:
 			# no adns query
 			self.continue_connect (True)
 		
-	def continue_connect (self, answer = None):
-		if not answer:
-			self.log ("DNS not found - %s" % self.address [0], "error")
-			return self.handle_close (704, "DNS Not Found")
+	def continue_connect (self, answer = None):		
+		answer = answer or adns.get (self.address [0], "A")
+		ipaddr = answer and answer [-1]["data"] or None
+		if DEBUG:
+			self.logger ('got DNS {rid:%s} %s %s' % (self.handler.request.meta.get ('req_id', -1), res, self.handler.request.uri), 'debug')
+		
+		if not ipaddr:
+			# for perventing recursion, check already close			
+			return self.handle_close (704)
 		
 		self.initialize_connection ()
-		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
-		
+		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)		
 		try:
 			if not adns.query:				
-				asynchat.async_chat.connect (self, self.address)
-				
+				asynchat.async_chat.connect (self, self.address)				
 			else:	
-				res = adns.get (self.address [0], "A")	
-				if res:
-					ip = res [-1]["data"]
-					if ip:
-						asynchat.async_chat.connect (self, (ip, self.address [1]))																		
-				else:
-					self.handle_close (704, "DNS Not Found")
-					
+				asynchat.async_chat.connect (self, (ipaddr, self.address [1]))
+									
 		except:	
-			self.handle_error ()
-	
-	def recv (self, buffer_size):		
-		self.set_event_time ()
+			self.handle_error (714)
+		
+	def recv (self, buffer_size):
 		try:
-			data = self.socket.recv(buffer_size)	
-			#print ("+++++DATA", len (data), repr (data [:40]))
+			data = self.socket.recv (buffer_size)			
 			if not data:
 				self.handle_close (700, "Connection closed unexpectedly in recv")
 				return b''
 			else:
+				self.set_event_time ()
 				return data		
 		except socket.error as why:
-			if why.errno in asyncore._DISCONNECTED:				
+			if why.errno in asyncore._DISCONNECTED:
 				self.handle_close (700, "Connection closed unexpectedly in recv")
 				return b''				
 			else:
 				raise
 	
 	def send (self, data):		
-		#print ("SEND", data)
-		self.set_event_time ()
 		try:
-			return self.socket.send (data)
+			numsent = self.socket.send (data)
+			if numsent:
+				self.set_event_time ()
+			return numsent	
+			
 		except socket.error as why:
 			if why.errno == EWOULDBLOCK:
 				return 0				
@@ -279,55 +306,31 @@ class AsynConnect (asynchat.async_chat):
 	def close_if_over_keep_live (self):
 		if time.time () - self.event_time > self.zombie_timeout:
 			self.disconnect ()
-	
-	def set_zombie_timeout (self, timeout = 10):
+		
+	def set_timeout (self, timeout = 10):
 		self.zombie_timeout = timeout
-	
-	def set_zombie_timeout_by_case (self):
-		if self.affluent or self.ready:
-			self.zombie_timeout = self.network_delay_timeout * 2
-		else:	
-			self.zombie_timeout = self.network_delay_timeout
-		
-	def set_timeout (self, a = 10, b = 10):
-		self.set_keep_alive_timeout (a)
-		self.set_network_delay_timeout (b)
-		
-	def set_network_delay_timeout (self, timeout = 10):
-		self.network_delay_timeout = timeout
-	
-	def set_keep_alive_timeout (self, timeout = 10):
-		self.keep_alive_timeout = timeout
 		
 	def handle_connect (self):
 		if hasattr (self.handler, "has_been_connected"):		
 			self.handler.has_been_connected ()
-		
-	def handle_read (self):
-		self.set_zombie_timeout_by_case ()		
-		asynchat.async_chat.handle_read (self)
-		
-	def handle_write (self):
-		self.set_zombie_timeout_by_case ()		
-		asynchat.async_chat.handle_write (self)
 			
 	def handle_expt (self):
-		self.logger ("socket panic", "fail")
-		self.handle_close (703, "Socket Panic")
+		#self.logger ("socket panic", "fail")
+		self.handle_close (703)
 	
-	def handle_error (self):
+	def handle_error (self, code = 701):
 		self.trace ()
-		self.handle_close(701, "Exception")
+		self.handle_close(code)
 	
 	def handle_timeout (self):
-		self.log ("socket timeout", "fail")
-		self.handle_close (702, "Zombie Timeout")
+		#self.log ("socket timeout", "fail")
+		self.handle_close (702)
 		
 	def handle_expt_event(self):
 		err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
 		if err != 0:
-			self.log ("Socket Error %d Occurred" % err, "warn")
-			self.handle_close (706, "Socket %d Error" % err)
+			#self.log ("Socket Error %d Occurred" % err, "warn")
+			self.handle_close (703, "Socket %d Error" % err)
 		else:
 			self.handle_expt ()
 	
@@ -343,7 +346,7 @@ class AsynConnect (asynchat.async_chat):
 		
 	# proxy POST need no init_send
 	def push (self, thing, init_send = True):
-		if type (thing) is bytes:
+		if isinstance(thing, (bytes, bytearray, memoryview)):
 			asynchat.async_chat.push (self, thing)
 		else:
 			self.push_with_producer (thing, init_send)	
@@ -357,14 +360,18 @@ class AsynConnect (asynchat.async_chat):
 		self.handler = None
 		self.close ()
 		
-	def handle_close (self, code = 700, msg = "Disconnected by Server"):
+	def handle_close (self, code = 700, msg = ""):
+		if code == 0: msg = ""
 		self.errcode = code
-		self.errmsg = msg
+		if msg:
+			self.errmsg = msg
+		else:
+			self.errmsg = respcodes.get (code, "Undefined Error")			
 		self.close ()
 							
 	def collect_incoming_data (self, data):
 		if not self.handler:
-			self.logger ("recv data but no hander, droping data %d" % len (data), "warn")
+			self.logger ("recv data but no handler, droping data %d" % len (data), "warn")
 			self.disconnect ()
 			return
 		self.handler.collect_incoming_data (data)
@@ -378,27 +385,28 @@ class AsynConnect (asynchat.async_chat):
 	
 	def disconnect (self):
 		# no error
-		self.handle_close (0, "")
+		self.handle_close (0)
 	
 	def reconnect (self):
-		self.disconnect ()
+		self.disconnect ()		
 		self.connect ()
 	
 	def set_proxy (self, flag = True):
 		self.proxy = flag
-		
-	def set_proxy_client (self, flag = True):
-		self.proxy_client = flag
-						
+	
 	def begin_tran (self, handler):
 		if self.__no_more_request:
-			raise SystemError ("Entered Shutdown Process")
+			return self.handle_close (705)
+		
+		# IMP: the reason why DNS error, _closed must be located here
+		self._closed = False
 		self.errcode = 0
 		self.errmsg = ""
 		
 		self.handler = handler
+		if DEBUG:
+			self.logger ('begin_tran {rid:%s} %s' % (self.handler.request.meta.get ('req_id', -1), self.handler.request.uri), 'debug')
 		self.set_event_time ()
-		self.proxy_client = False
 		
 		if self.connected:
 			self.close_if_over_keep_live () # check keep-alive
@@ -416,19 +424,22 @@ class AsynConnect (asynchat.async_chat):
 
 
 class AsynSSLConnect (AsynConnect):	
-	ac_out_buffer_size = 65536
-	ac_in_buffer_size = 65536
+	ac_negotiate_http2 = True
+	
+	def negotiate_http2 (self, flag):
+		self.ac_negotiate_http2 = flag
 		
 	def handshake (self):
 		if not self._handshaking:
 			err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
 			if err != 0:
-				raise socket.error(err, _strerror(err))								
+				raise socket.error(err, _strerror(err))
+				
 			ssl_context = create_urllib3_context(ssl_version=resolve_ssl_version(None), cert_reqs=resolve_cert_reqs(None))
-			
-			try: ssl_context.set_alpn_protocols (H2_PROTOCOLS)
-			except AttributeError: ssl_context.set_npn_protocols (H2_PROTOCOLS)								
-			self.socket = ssl_context.wrap_socket (self.socket, do_handshake_on_connect = False, server_hostname = self.address [0])
+			if self.ac_negotiate_http2:
+				try: ssl_context.set_alpn_protocols (H2_PROTOCOLS)
+				except AttributeError: ssl_context.set_npn_protocols (H2_PROTOCOLS)								
+			self.socket = ssl_context.wrap_socket (self.socket, do_handshake_on_connect = False, server_hostname = self.address [0])			
 			self._handshaking = True
 			
 		try:
@@ -437,32 +448,41 @@ class AsynSSLConnect (AsynConnect):
 			if why.args [0] in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
 				return False
 			raise ssl.SSLError(why)
-		
+			
 		try: self._proto = self.socket.selected_alpn_protocol()
 		except (AttributeError, NotImplementedError): 
 			try: self._proto = self.socket.selected_npn_protocol()
 			except (AttributeError, NotImplementedError): pass
 
-		self._handshaked = True		
+		self._handshaked = True
 		return True
 							
-	def handle_connect_event (self):	
-		if not self._handshaked and not self.handshake ():
+	def handle_connect_event (self):
+		try:
+			if not self._handshaked and not self.handshake ():
+				return
+		except:
+			self.handle_error (713)
 			return
+					
 		# handshaking done
 		self.handle_connect()
 		self.connected = True
 		
 	def recv (self, buffer_size):
-		self.set_event_time ()
+		if self._closed:
+			# usually handshaking failure, already handled exception
+			return
+			
 		try:
 			data = self.socket.recv (buffer_size)			
 			if not data:				
 				self.handle_close (700, "Connection closed unexpectedly")
 				return b''
 			else:				
+				self.set_event_time ()
 				return data
-
+			
 		except ssl.SSLError as why:
 			if why.errno == ssl.SSL_ERROR_WANT_READ:
 				try: 
@@ -479,9 +499,15 @@ class AsynSSLConnect (AsynConnect):
 				raise
 
 	def send (self, data):
-		self.set_event_time ()
+		if self._closed:
+			# usually handshaking failure, already handled exception
+			return
+			
 		try:
-			return self.socket.send (data)			
+			numsent = self.socket.send (data)			
+			if numsent:
+				self.set_event_time ()
+			return numsent	
 
 		except ssl.SSLError as why:
 			if why.errno == ssl.SSL_ERROR_WANT_WRITE:
@@ -494,22 +520,19 @@ class AsynSSLConnect (AsynConnect):
 
 
 class AsynSSLProxyConnect (AsynSSLConnect, AsynConnect):
-	ac_out_buffer_size = 65536
-	ac_in_buffer_size = 65536
-	
 	def handle_connect_event (self):
 		if self.established:
 			AsynSSLConnect.handle_connect_event (self)
 		else:	
 			AsynConnect.handle_connect_event (self)
 	
-	def recv (self, buffer_size):		
+	def recv (self, buffer_size):	
 		if self._handshaked or self._handshaking:
 			return AsynSSLConnect.recv (self, buffer_size)				
 		else:
 			return AsynConnect.recv (self, buffer_size)
 			
-	def send (self, data):		
+	def send (self, data):	
 		if self._handshaked or self._handshaking:
 			return AsynSSLConnect.send (self, data)
 		else:
