@@ -6,10 +6,30 @@ try:
 except ImportError:
 	from urlparse import urlparse	
 import copy
-from aquests.protocols.http2 import H2_PROTOCOLS
+from ..protocols.http2 import H2_PROTOCOLS
+from ..protocols import http2
 
-PROTO_LOADBALANCE = H2_PROTOCOLS
+PROTO_CONCURRENT_STREAMS = H2_PROTOCOLS
 
+def select_channel (asyncons):
+	asyncon = None
+	_asyncons = []
+	for each in asyncons:
+		if each.isactive ():
+			continue
+		if not each.isconnected ():
+			_asyncons = [(each, 0)]
+			break
+		if each.handler is not None:			
+			_asyncons.append ((each, each.handler.jobs ()))
+	
+	if _asyncons:
+		_asyncons.sort (key = lambda x: x [1])
+		if _asyncons [0][1] < http2.MAX_HTTP2_CONCURRENT_STREAMS:
+			asyncon = _asyncons [0][0]
+	
+	return asyncon
+	
 
 class SocketPool:
 	object_timeout = 120
@@ -105,7 +125,7 @@ class SocketPool:
 				del self.__socketfarm [serverkey]
 				
 		self.__last_maintern = time.time ()
-	
+				
 	def _get (self, serverkey, server, *args):
 		asyncon = None
 		self.lock.acquire ()
@@ -121,15 +141,16 @@ class SocketPool:
 					self.__socketfarm [serverkey][id (asyncon)] = asyncon
 					
 				else:		
-					proto = self.__protos.get (serverkey)
 					asyncons = list(self.__socketfarm [serverkey].values ())
-					if proto in PROTO_LOADBALANCE:
-						random.shuffle (asyncons)
-												
-					for each in asyncons:	
-						if not each.isactive ():
-							asyncon = each
-							break
+					#print (self.__protos.get (serverkey), PROTO_CONCURRENT_STREAMS)
+					if self.__protos.get (serverkey) in PROTO_CONCURRENT_STREAMS:
+						asyncon = select_channel (asyncons)
+						#print (len (asyncons), asyncon)
+					else:
+						for each in asyncons:								
+							if not each.isactive ():
+								asyncon = each
+								break
 					
 					if not asyncon:
 						asyncon = self.create_asyncon (server, *args)
@@ -179,9 +200,15 @@ class SocketPool:
 				
 	def get (self, uri):	
 		scheme, server, script, params, qs, fragment = urlparse (uri)
-		serverkey = "%s://%s" % (scheme, server)		
+		serverkey = "%s://%s" % (scheme, server)	
 		return self._get (serverkey, server, scheme)
-		
+	
+	def noop (self):
+		with self.lock:
+			for server in list(self.__socketfarm.keys ()):					
+				for asyncon in list(self.__socketfarm [server].values ()):
+					asyncon.set_event_time ()
+			
 	def cleanup (self):
 		self.lock.acquire ()
 		try:
@@ -189,6 +216,9 @@ class SocketPool:
 				for server in list(self.__socketfarm.keys ()):					
 					for asyncon in list(self.__socketfarm [server].values ()):
 						asyncon.disconnect ()
+						del asyncon
+				self.__socketfarm = {}
+				self.__protos = {}
 			finally:
 				self.lock.release ()
 		except:
@@ -202,8 +232,12 @@ def create (logger):
 	if pool is None:
 		pool = SocketPool (logger)
 
-def get (uri):	
+def get (uri):
 	return pool.get (uri)
 		
 def cleanup ():	
 	pool.cleanup ()
+
+def noop ():
+	pool.noop ()
+	
\ No newline at end of file
