@@ -15,11 +15,22 @@ def trace ():
 		v = str (v)
 	except UnicodeEncodeError:
 		pass
-	return "%s %s %s" % (t, v, tbinfo)
+	return "%s %s Traceback: %s" % (t, v, tbinfo)
 		
 def now (detail = 1):
 	if detail: return time.strftime("%Y.%m.%d %H:%M:%S", time.localtime(time.time()))
 	else: return time.strftime("%Y%m%d", time.localtime(time.time()))
+
+def ago (t):
+	diff = time.time () - t
+	if diff < 3600:
+		return "%2.1f minutes ago" % (diff/60)
+	elif diff < 3600	 * 24:
+		return "%2.1f hours ago" % (diff/3600)
+	elif diff < 3600	 * 72:
+		return "%2.1f days ago" % (diff/(3600 * 24))	
+	return time.strftime ("%Y-%m-%d", time.localtime (t))
+
 	
 class base_logger:
 	def __init__(self, out, cacheline = 100, flushnow = 0):
@@ -33,7 +44,7 @@ class base_logger:
 	def set_filter (self, *types):	
 		self.filter = types
 		
-	def __call__(self, line, type="", name=""):
+	def __call__(self, line, type="info", name=""):
 		return self.log (line, type, name)
 	
 	def write (self, line, type="", name=""):
@@ -91,14 +102,20 @@ class base_logger:
 	def read (self):
 		return self.__cache
 		
-
 	
 class screen_logger (base_logger):
 	def __init__ (self, cacheline = 200, flushnow = 1):
 		base_logger.__init__(self, sys.stdout, cacheline, flushnow)
 	
-	# do nothing
-	def close (self): pass
+	def log (self, line, type = "info", name = ""):
+		if type.startswith ("expt"):
+			line = trace ().replace ("] [", "\n  - ")
+			line = line.replace ("Traceback: [", "\n  -----------\n  + Traceback\n  ===========\n  - ")
+			line = line [:-1] + "\n  -----------"
+		base_logger.log (self, line, type, name)			
+	
+	def close (self): 
+		pass
 
 
 class null_logger (screen_logger):
@@ -130,6 +147,7 @@ class pipe_logger (screen_logger):
 
 
 class rotate_logger (base_logger):
+	PRESERVE_MAX = 30
 	def __init__(self, base, surfix = '', freq = "daily", cacheline = 200, flushnow = 0):
 		self.base = base
 		self.surfix = surfix
@@ -169,9 +187,9 @@ class rotate_logger (base_logger):
 			if file.find (self.surfix + "-") != 0: continue 			
 			dlogs.append (file) 
 		
-		if len (dlogs) <= 100: return
+		if len (dlogs) <= self.PRESERVE_MAX: return
 		dlogs.sort ()
-		for file in dlogs [:-100]:
+		for file in dlogs [:-self.PRESERVE_MAX]:
 			try: os.remove (os.path.join (self.base, file))
 			except: pass
 		
@@ -246,7 +264,11 @@ class multi_logger (base_logger):
 			self.loggers.append (logger)
 			
 		base_logger.__init__ (self, None, cacheline, flushnow)
-		
+	
+	def rotate (self):
+		for logger in self.loggers:
+			hasattr (logger, 'rotate') and logger.rotate ()
+			
 	def log (self, line, type="info", name=""):
 		if self.filter and type not in self.filter:
 			return line
