@@ -16,6 +16,8 @@ _last_maintern = 0
 _maintern_interval = 3.0
 _killed_zombies = 0
 _select_errors = 0
+_poll_count = 0
+_polling = 0
 	
 class Maintern:
 	def __init__ (self):
@@ -88,6 +90,7 @@ def loop (timeout = 30.0):
 	global _shutdown_phase
 	global _shutdown_timeout
 	global _exit_code
+	global _polling
 	global maintern
 	
 	if maintern is None:
@@ -96,14 +99,17 @@ def loop (timeout = 30.0):
 	_shutdown_phase = 0
 	_shutdown_timeout = 30
 	_exit_code = 0	
-
+	_polling = 1
+	
 	try: 
 		lifetime_loop(timeout)
 	except KeyboardInterrupt:
 		graceful_shutdown_loop()
 	else:
-		graceful_shutdown_loop()	
-
+		graceful_shutdown_loop()
+	
+	_polling = 0
+	
 if hasattr(select, 'poll'):
 	poll_fun = asyncore.poll2
 else:
@@ -133,43 +139,49 @@ def remove_notsocks (map):
 	return killed
 	
 
-def poll_fun_wrap (timeout, map):
+def poll_fun_wrap (timeout, map = None):
+	if map is None:
+		map = asyncore.socket_map
+		
 	try:		
 		poll_fun (timeout, map)
 	
 	except select.error as why:
-		# WSAENOTSOCK		
+		# WSAENOTSOCK
 		remove_notsocks (map)
 	
-	except ValueError:
+	except ValueError:		
 		# negative file descriptor, testing all sockets
-		killed = remove_notsocks (map)
+		killed = remove_notsocks (map)		
 		if not killed:
 			# too many file descriptors in select(), divide and conquer
-			half = len (map) / 2
+			half = int (len (map) / 2)
 			tmap = {}
-			cc = 0
+			cc = 0			
 			for k, v in list(map.items ()):
 				tmap [k] = v
 				cc += 1
-				if cc == half:
+				if cc == half:					
 					poll_fun_wrap (timeout, tmap)
 					tmap = {}
-					
-		poll_fun_wrap (timeout, tmap)
+			poll_fun_wrap (timeout, tmap)
 
 
-def lifetime_loop (timeout = 30.0):
+def lifetime_loop (timeout = 30.0, count = 0):
 	global _last_maintern
 	global _maintern_interval
-				
+
 	map = asyncore.socket_map
+	loop = 0
 	while map and _shutdown_phase == 0:
 		poll_fun_wrap (timeout, map)
 		now = time.time()
 		if (now - _last_maintern) > _maintern_interval:
 			maintern (now)
 			_last_maintern = time.time ()
+		loop += 1
+		if count and loop > count:
+			break	
 		
 def graceful_shutdown_loop ():
 	global _shutdown_phase
@@ -191,13 +203,7 @@ def graceful_shutdown_loop ():
 					obj.handle_error()
 					
 		if veto and time_in_this_phase < _shutdown_timeout:
-			try:
-				poll_fun(timeout, map)
-			except select.error as why:
-				if os.name == "nt":
-					if whay.args [0] == WSAENOTSOCK: # sometimes postgresql connection forcely closed
-						remove_notsocks (map)
-					
+			poll_fun_wrap (timeout, map)					
 		else:
 			_shutdown_phase += 1
 			timestamp = time.time()
