@@ -15,7 +15,7 @@ defaults = Base.defaults
 Base.DiscoverNameServers ()
 
 class async_dns (asyncore.dispatcher_with_send):
-	zombie_timeout = 3
+	zombie_timeout = 10
 	
 	def __init__ (self, servers, request, args, callback, logger, debug_level):
 		self.servers = servers
@@ -38,7 +38,6 @@ class async_dns (asyncore.dispatcher_with_send):
 		
 		asyncore.dispatcher_with_send.__init__ (self)		
 		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)				
-		self.ac_out_buffer = self.request
 				
 		try:
 			self.connect (self.addr)
@@ -46,27 +45,24 @@ class async_dns (asyncore.dispatcher_with_send):
 			self.handle_error ()
 	
 	def __repr__ (self):
-		return "<async_dns: %s>" % self.args ["name"]
+		return "<async_dns: %s>" % self.qname
 				
 	def trace (self):
-		self.logger.trace ()
+		self.logger.trace (self.qname)
 	
 	def log_info (self, line, level = 'info'):
-		self.log ("[%s] %s" % (level, line))
+		self.log ("[%s:%s] %s" % (level, self.qname, line))
 	
 	def log (self, line):
-		self.logger (line)	
+		self.logger (line)
 	
 	def create_socket (self, family, type):
-		if hasattr (socket, "_no_timeoutsocket"):
-			sock_class = socket._no_timeoutsocket
-		else:
-			sock_class = socket.socket
-			
+		sock_class = socket.socket			
 		self.family_and_type = family, type
 		self.socket = sock_class (family, type)
 		self.socket.setblocking (0)
 		self._fileno = self.socket.fileno ()
+		self._timeouted = 0
 		self.add_channel()
 		
 	def handle_error (self):
@@ -75,16 +71,26 @@ class async_dns (asyncore.dispatcher_with_send):
 		self.close ()
 	
 	def handle_timeout (self):
+		if self.debug_level: 
+			self.log_info ('DNS query timeout %s' % self.callback, 'error')
+			self._timeouted = 1
 		self.handle_close ()
 					
 	def handle_connect (self):	
 		self.send (self.request)
 		
 	def handle_write (self):
-		self.socket.shutdown (1)
+		if not self.closed:
+			self.socket.shutdown (1)
 		
 	def handle_read (self):
-		data = self.recv (4096)
+		try:
+			data = self.recv (4096)
+		except BlockingIOError:
+			return
+		except OSError as why:
+			self.handle_close()
+			return
 		self.ac_in_buffer += data		
 		
 	def handle_expt (self):
@@ -94,8 +100,8 @@ class async_dns (asyncore.dispatcher_with_send):
 		if self.closed:
 			return
 		self.closed = True	
-		asyncore.dispatcher_with_send.close (self)
-		self.callback (self.servers, self.request, self.args, self.ac_in_buffer)
+		asyncore.dispatcher_with_send.close (self)		
+		self.callback (self.servers, self.request, self.args, self.ac_in_buffer, self._timeouted)
 			
 	def handle_close (self):
 		self.close ()
@@ -106,7 +112,7 @@ class Request:
 		if type (server) is bytes:
 			server = [server]
 		
-		defaults ["server"] += server
+		defaults ["server"] += server		
 		self.logger = logger
 		self.debug_level = debug_level		
 		
@@ -133,6 +139,7 @@ class Request:
 		opcode = args ['opcode']
 		rd = args ['rd']
 		server = args ['server'][:]
+		#server = ['127.0.0.1:6000']
 		
 		if type(args['qtype']) in (bytes, str):
 			try:
@@ -159,52 +166,60 @@ class Request:
 		server = [(x, args ["port"]) for x in server]
 		async_dns (server, request, args, self.processReply, self.logger, self.debug_level)
 			
-	def processReply(self, server, request, args, data):		
-		exception = 0
-		
-		try:
-			if not data:
-				raise Base.DNSError('%s, no working nameservers found' % args ['name'])
-		
-			if args ["protocol"] == "tcp":
-				header = data [:2]
-				if len (header) < 2:
-					raise Base.DNSError('%s, EOF' % args ['name'])
-				count = Lib.unpack16bit(header)		
-				reply = data [2: 2 + count]					
-				if len (reply) != count:
-					raise Base.DNSError('%s, incomplete reply' % args ['name'])						
-			
-			else:
-				reply = data
-			
-		except:
-			if server:				
-				async_dns (server, request, args, self.processReply, self.logger, self.debug_level)
-				return
-				
-			else:
-				reply = None
-		
-		if reply is None:		
+	def processReply (self, server, request, args, data, timeouted):
+		if timeouted:
+			# for retrying
 			answers = []
 			
-		else:
-			try:	
-				u = Lib.Munpacker(reply)
-				r = Lib.DnsResult(u, args)
-				r.args = args
-				answers = r.answers
+		else:	
+			# do not query for 1 minutes
+			not_found = [{'err': True, "name": args ['name'].decode ('utf8'), "data": None, "typename": args ["qtype"], 'ttl': 60}]
 				
-			except:
-				if self.debug_level: self.logger.trace ()
-				answers = []	
-
+			try:
+				if not data:
+					raise Base.DNSError('%s, no working nameservers found' % args ['name'])
+			
+				if args ["protocol"] == "tcp":
+					header = data [:2]
+					if len (header) < 2:
+						raise Base.DNSError('%s, EOF' % args ['name'])
+					count = Lib.unpack16bit(header)		
+					reply = data [2: 2 + count]
+					if len (reply) != count:
+						raise Base.DNSError('%s, incomplete reply' % args ['name'])
+				
+				else:
+					reply = data
+				
+			except:					
+				if server:
+					async_dns (server, request, args, self.processReply, self.logger, self.debug_level)
+					return
+					
+				else:
+					reply = None
+			
+			if reply is None:
+				answers = not_found
+				
+			else:
+				try:	
+					u = Lib.Munpacker(reply)
+					r = Lib.DnsResult(u, args)
+					r.args = args
+					answers = r.answers
+				except:
+					self.logger.trace ()
+					answers = not_found
+					
+			if not answers:
+				answers = not_found
+		
 		callback = args.get ("callback", None)
+		#self.logger ('DNS callback %s' % callback)
 		if callback:
 			if type (callback) != type ([]):
 				callback = [callback]
-			
 			for cb in callback:
 				cb (answers)
 			
@@ -213,7 +228,6 @@ if __name__	== "__main__":
 	from aquests.lib import logger
 	import pprint
 	f = Request (logger.screen_logger ())
-	f.req ("www.openfos.com", protocol = "tcp", callback = pprint.pprint, qtype="a")
 	f.req ("www.yahoo.com", protocol = "tcp", callback = pprint.pprint, qtype="a")
 	f.req ("www.hungryboarder.com", protocol = "tcp", callback = pprint.pprint, qtype="a")
 	f.req ("www.alexa.com", protocol = "tcp", callback = pprint.pprint, qtype="a")
