@@ -1,108 +1,41 @@
 from . import response as http_response
-from aquests.protocols.http2 import request_handler as http2_request_handler
-from . import base_request_handler
-from aquests.client import asynconnect
+from ..http2 import request_handler as http2_request_handler
+from . import base_request_handler, http_auth, respcodes
+from ...client import asynconnect
 import base64
-from aquests.protocols.http import http_util
 from hashlib import md5
-from base64 import b64encode
 import os
 from hyperframe.frame import SettingsFrame
-from aquests.protocols.http2 import H2_PROTOCOLS
+from ..http2 import H2_PROTOCOLS
+from ...client import socketpool
+from ...lib.cbutil import tuple_cb
 
-class Authorizer:
-	def __init__ (self):
-		self.db = {}
+class ProtocolSwitchError (Exception):
+	pass
+	
 	
-	def get (self, netloc, auth, method, uri, data):
-		if netloc not in self.db:
-			return ""
-			
-		infod = self.db [netloc]
-		if infod ["meth"] == "basic":
-			return "Basic " + base64.encodestring ("%s:%s" % auth) [:-1]
-		elif infod ["meth"] == "bearer":
-			return "Bearer " + auth [0]
-		else:
-			infod ["nc"] += 1
-			hexnc = hex (infod ["nc"])[2:].zfill (8)
-			infod ["cnonce"] = http_util.md5uniqid ()
-			
-			A1 = md5 (("%s:%s:%s" % (auth [0], infod ["realm"], auth [1])).encode ("utf8")).hexdigest ()
-			if infod ["qop"] == "auth":
-				A2 = md5 (("%s:%s" % (method, uri)).encode ("utf8")).hexdigest ()
-			elif type (data) is bytes:
-				entity = md5 (data).hexdigest ()
-				A2 = md5 (("%s:%s" % (method, uri)).encode ("utf8")).hexdigest ()
-			else:
-				return # sorry data is not bytes
-						
-			Hash = md5 (("%s:%s:%s:%s:%s:%s" % (
-				A1,
-				infod ["nonce"],
-				hexnc,
-				infod ["cnonce"],
-				infod ["qop"],
-				A2
-				)).encode ("utf8")
-			).hexdigest ()
-			
-			return (
-				'Digest username="%s", realm="%s", nonce="%s", '
-				'uri="%s", response="%s", opaque="%s", qop=%s, nc=%s, cnonce="%s"' % (
-					auth [0], infod ["realm"], infod ["nonce"], uri, Hash, 
-					infod ["opaque"], infod ["qop"], hexnc, infod ["cnonce"]
-				)
-			)
-			
-	def set (self, netloc, authreq, auth):
-		if auth is None:
-			return
-		
-		amethod, authinfo = authreq.split (" ", 1)		
-		infod = {"meth": amethod.lower ()}
-		infod ["nc"] = 0
-		for info in authinfo.split (","):
-			k, v = info.strip ().split ("=", 1)
-			if not v: return self.get_www_authenticate ()
-			if v[0] == '"': v = v [1:-1]
-			infod [k]	 = v
-		
-		if "qop" in infod:
-			qop = list (map (lambda x: x.strip (), infod ["qop"].split (",")))
-			if "auth" in qop:
-				infod ["qop"] = "auth"
-			else:
-				infod ["qop"] = "auth-int"
-				
-		self.db [netloc] = infod
-		
-
-authorizer = Authorizer ()
-
 class RequestHandler (base_request_handler.RequestHandler):
+	FORCE_HTTP_11 = False	
+	
 	def __init__ (self, asyncon, request, callback, connection = "keep-alive"):
 		self.asyncon = asyncon
 		self.wrap_in_chunk = False
 		self.end_of_data = False
-		self.expect_disconnect = False		
+		self.expect_disconnect = False
 		
 		self.request = request
 		self.callback = callback
 		base_request_handler.RequestHandler.__init__ (self, request.logger)
 		self.connection = connection				
 		
-		self.expect_disconnect = False
 		self.retry_count = 0
-		self.reauth_count = 0
+		self.http2_handler = None
 		
 		self.buffer = b""	
 		self.response = None
+		self.callbacked = 0
 		
-		self._ssl = False
-		if isinstance (self.asyncon, asynconnect.AsynSSLConnect):
-			self._ssl = True
-			
+		self._ssl = isinstance (self.asyncon, asynconnect.AsynSSLConnect)			
 		self.method, self.uri = (
 			self.request.get_method (),			
 			self.asyncon.is_proxy () and self.request.uri or self.request.path
@@ -120,16 +53,7 @@ class RequestHandler (base_request_handler.RequestHandler):
 	#------------------------------------------------
 	# handler must provide these methods
 	#------------------------------------------------
-	def get_http_auth_header (self, data = b""):
-		auth = self.request.get_auth ()
-		if auth:
-			uri = self.asyncon.is_proxy () and self.request.uri or self.request.path
-			auth_header = authorizer.get (self.request.get_address (), auth, self.method, uri, data)
-			if auth_header is None:
-				raise AssertionError ("Unknown authedentification method")
-			return auth_header
-	
-	def rebuild_http2_headers (self, headers, headers_1x, payload):
+	def rebuild_http2_headers (self, headers, headers_1x):
 		content_encoding = None
 		for k, v in headers_1x:
 			k = k.lower ()
@@ -164,19 +88,19 @@ class RequestHandler (base_request_handler.RequestHandler):
 		else:
 			hc = {}
 			
-		payload = self.request.get_payload ()	
-		auth_header = self.get_http_auth_header (payload)
-		if auth_header:
-			hc ["Authorization"] = auth_header
-		
+		auth_header = http_auth.authorizer.make_http_auth_header (self.request, self.asyncon.is_proxy ())
 		if http_version == "2.0":
 			headers = [
 				(":method", self.method),
 				(":path", self.uri),
 				(":scheme", self._ssl and "https" or "http")
 			]
-			return self.rebuild_http2_headers (headers, self.request.get_headers (), payload)			
+			if auth_header:
+				headers.append (("authorization", auth_header))
+			return self.rebuild_http2_headers (headers, self.request.get_headers ())			
 		
+		if auth_header:
+			hc ["Authorization"] = auth_header
 		headers = list (hc.items ()) + self.request.get_headers ()
 		self.header = ["%s: %s" % x for x in headers]
 		req = ("%s %s HTTP/%s\r\n%s\r\n\r\n" % (
@@ -197,13 +121,16 @@ class RequestHandler (base_request_handler.RequestHandler):
 		return [self.get_request_header (http_version, upgrade), payload]	
 		
 	def collect_incoming_data (self, data):
+		if not data:
+			self.end_of_data = True
+			return
 		if not self.response or self.asyncon.get_terminator () == b"\r\n":
 			self.buffer += data
 		else:			
 			self.response.collect_incoming_data (data)
 		
 	def found_terminator (self):
-		if self.response:
+		if self.response:			
 			if self.end_of_data:
 				return self.found_end_of_body ()
 			
@@ -231,32 +158,24 @@ class RequestHandler (base_request_handler.RequestHandler):
 						
 		else:
 			self.expect_disconnect = False
-			try:
-				self.create_response ()
-			except:					
-				# I don't know why handle exception here.
-				# If not, recv data continuously T.T
-				self.asyncon.handle_error ()
+			self.create_response ()
+			if not self.response or isinstance (self.response, http_response.FailedResponse):
 				return
-					
-			# 100 Continue etc. try recv continued header
-			if self.response is None:
-				return				
 			
 			if self.used_chunk ():
 				self.wrap_in_chunk = True
 				self.asyncon.set_terminator (b"\r\n") #chunked transfer
 			
-			else:			
+			else:
+				clen = 0 # no transfer-encoding, no content-lenth	
 				try:
 					clen = self.get_content_length ()
 				except TypeError:
 					if self.will_be_close ():
-						clen = ""
 						self.expect_disconnect = True
-					else:
-						clen = 0 # no transfer-encoding, no content-lenth												
-				
+						if self.response.get_header ("content-type"):
+							clen = None							
+						
 				if clen == 0:
 					return self.found_end_of_body ()
 					
@@ -265,115 +184,152 @@ class RequestHandler (base_request_handler.RequestHandler):
 	def create_response (self):
 		buffer, self.buffer = self.buffer, b""
 		try:
-			self.response = http_response.Response (self.request, buffer.decode ("utf8"))
+			self.response = http_response.Response (self.request, buffer.decode ("utf8"))			
+			if self.handle_response_code ():
+				return
+		except ProtocolSwitchError:
+			return self.asyncon.handle_error (716)
 		except:
 			self.log ("response header error: `%s`" % repr (buffer [:80]), "error")
-			self.asyncon.handle_close (708, "Response Header Error")
-		else:	
-			self.is_continue_response ()
+			return self.asyncon.handle_error (715)			
+			
+		accepts = self.request.get_header ("accept", '')
+		if accepts in ("", "*/*"):
+			return
 		
-	def is_continue_response (self):	
+		ct = self.response.get_header ("content-type", 'text/html').split (";")[0].strip ()
+		try: 
+			mct, sct = ct.split ("/")
+		except:
+			return self.asyncon.handle_error (717)
+			
+		for accept in accepts.split (","):
+			acctype = accept.split (";")[0].strip ()
+			try: mtype, stype = acctype.split ("/", 1)
+			except ValueError: continue
+			if mtype in ("*", mct) and stype in ("*", sct):
+				return
+		
+		self.asyncon.handle_close (718)		
+		
+	def handle_response_code (self):
 		# default header never has "Expect: 100-continue"
 		# ignore, wait next message	
 		if self.response.code == 100:
 			self.asyncon.set_terminator (b"\r\n\r\n")
+			self.response = None			
+			return 1
+			
+		elif self.response.code == 101 and self.response.get_header ("Upgrade") == "h2c":	# swiching protocol		
+			self.asyncon._proto = "h2c"
 			self.response = None
-			return True
-		elif self.response.code == 101:	# swiching protocol
-			if self.response.get_header ("Upgrade") == "h2c":
-				self.asyncon._proto = "h2c"
-				self.response = None
-				self.switch_to_http2 ()				
-				return True
-		return False
-		
-	def found_end_of_body (self):	
-		if self.response:
-			self.response.done ()
-		if self.handled_http_authorization ():
-			return
-		if self.will_be_close ():
-			self.asyncon.disconnect ()			
-		self.close_case_with_end_tran ()
+			try:				
+				self.switch_to_http2 ()
+			except:	
+				raise ProtocolSwitchError
+			return 1
+		return 0	
 	
 	def handled_http_authorization (self):
 		if self.response.code != 401:
-			return 0 #pass
-			
-		if self.reauth_count > 0:
-			self.asyncon.handle_close (710, "Authorization Failed")
-			return 1 # abort
-		
-		self.reauth_count = 1		
+			return
+						
+		if self.request.reauth_count > 0:
+			return
+
 		try: 
-			authorizer.set (self.request.get_address (), self.response.get_header ("WWW-Authenticate"), self.request.get_auth ())					
+			http_auth.authorizer.save_http_auth_header (self.request, self.response)
 		except:
 			self.trace ()
-			self.asyncon.handle_close (711, "Unknown Authedentification Method")
-			return 1 # abort			
-		else:
-			self.handle_request ()
-			return 1
+				
+	def found_end_of_body (self):
+		if self.response:
+			self.response.done ()		
 		
-		return 0 #pass
-			
-	def connection_closed (self, why, msg):
-		is_real_asyncon = hasattr (self.asyncon, "address")
-		
-		if not is_real_asyncon: # http2
-			self.response = http_response.FailedResponse (why, msg, self.request)
-			if self.callback:
-				self.callback (self)
-			return	
-			
+		self.handled_http_authorization ()			
+		if self.will_be_close ():
+			self.asyncon.disconnect ()		
+		self.close_case_with_end_tran ()
+				
+	def connection_closed (self, why, msg):		
 		if self.response and self.expect_disconnect:
 			self.close_case ()
 			return
 		
-		# possibly disconnected cause of keep-alive timeout		
-		if why == 700 and self.response is None and self.retry_count == 0:
-			self.retry_count = 1					
-			# if not exists, fake asyncon
-			self.handle_request ()
-			return
+		# possibly disconnected cause of keep-alive timeout
+		# but works only HTTP 1.1
+		if not self.http2_handler and why == 700 and self.response is None and self.retry_count == 0:
+			self.retry_count = 1
+			self.handle_rerequest ()
+			return True
 		
 		self.response = http_response.FailedResponse (why, msg, self.request)
 		if hasattr (self.asyncon, "begin_tran"):
 			self.close_case ()
-	
+			
 	def close_case_with_end_tran (self):
 		self.asyncon.end_tran ()
 		self.close_case ()
 		
+	def handle_callback (self):
+		if self.callbacked:
+			return
+		tuple_cb (self, self.callback)
+		self.callbacked = 1
+			
 	def close_case (self):
 		if self.asyncon:
-			self.asyncon.handler = None # unlink back ref.
-		if self.callback:
-			self.callback (self)
+			self.asyncon.handler = None # unlink back ref.		
+		self.handle_callback ()
 	
-	def switch_to_http2 (self):
-		http2_request_handler.RequestHandler (self)		
+	def switch_to_http2 (self):		
+		if self.http2_handler is None:			
+			self.http2_handler = http2_request_handler.RequestHandler (self)			
+		else:
+			self.http2_handler.handle_request (self)	
 		
 	def has_been_connected (self):
 		if self._ssl or self.request.initial_http_version == "2.0":
 			if self.request.initial_http_version == "2.0":
-				self.asyncon.set_proto ("h2c")
-			if self.asyncon._proto in H2_PROTOCOLS:
+				self.asyncon.set_proto ("h2c")			
+			if self.asyncon._proto in H2_PROTOCOLS:				
 				self.switch_to_http2 ()
 			else:
 				for data in self.get_request_buffer ("1.1", False):
-					self.asyncon.push (data)				
-					
+					self.asyncon.push (data)
+	
+	def handle_rerequest (self):
+		# init for redirexting or reauth
+		self.response = None
+		self.end_of_data = False
+		self.buffer = b''
+		self.callbacked = 0
+		self.handle_request ()
+						
 	def handle_request (self):
+		if self.asyncon.isconnected () and self.asyncon.get_proto ():			
+			return self.switch_to_http2 ()
+		
 		self.buffer, self.response = b"", None
-		self.asyncon.set_terminator (b"\r\n\r\n")		
+		self.asyncon.set_terminator (b"\r\n\r\n")
+		
 		if (self.asyncon.connected) or not (self._ssl or self.request.initial_http_version == "2.0"):
-			# IMP: if already connected, it means not http2			
-			for data in self.get_request_buffer ("1.1", not self.asyncon.connected and True or False):			
-				self.asyncon.push (data)		
+			# IMP: if already connected, it means not http2
+			upgrade = True
+			if self.FORCE_HTTP_11:
+				upgrade = False
+			elif self.asyncon.isconnected ():
+				upgrade = False
+			
+			for data in self.get_request_buffer ("1.1", upgrade):				
+				self.asyncon.push (data)
+				
+		if self._ssl and self.FORCE_HTTP_11 and self.request.initial_http_version != "2.0":
+			self.asyncon.negotiate_http2 (False)
+			
 		self.asyncon.begin_tran (self)
 	
-	def will_be_close (self):		
+	def will_be_close (self):
 		if self.connection == "close": #server misbehavior ex.paxnet
 			return True
 				
@@ -397,8 +353,8 @@ class RequestHandler (base_request_handler.RequestHandler):
 					
 					if k.strip () == "timeout": 
 						timeout = int (v)
-						if timeout < self.asyncon.keep_alive_timeout:
-							self.asyncon.set_keep_alive_timeout (timeout)
+						if timeout < self.asyncon.zombie_timeout:
+							self.asyncon.set_timeout (timeout)
 					elif k.strip () == "max" and int (v) == 0:
 						close_it = True
 								
