@@ -3,6 +3,8 @@ try:
 	from h2.exceptions import FlowControlError
 except ImportError:
 	pass	
+from aquests.lib import producers
+import asynchat
 
 class h2header_producer:
 	def __init__ (self, stream_id, headers, producer, encoder, lock):
@@ -16,7 +18,10 @@ class h2header_producer:
 				end_stream = producer is None
 			)
 			self.data_to_send = encoder.data_to_send ()
-			
+	
+	def get_size (self):
+		return 0
+		
 	def __repr__ (self):
 		return "<h2header_producer stream_id:%d>" % (self.__stream_id,)
 	
@@ -25,8 +30,8 @@ class h2header_producer:
 		return data_to_send
 
 
-class h2data_producer:
-	BUFFER_SIZE = 4096	
+class h2frame_producer:
+	SIZE_BUFFER = 16384
 	
 	def __init__ (self, stream_id, depends_on, weight, producer, encoder, lock):
 		self.stream_id = stream_id
@@ -43,28 +48,31 @@ class h2data_producer:
 		if hasattr (producer, "ready"):
 			self.ready = producer.ready
 			producer.ready = None
-				
+	
+	def get_size (self):			
+		return self.producer.get_size ()
+		
 	def __repr__ (self):
-		return "<h2data_producer stream_id:%d, weight:%d, depends_on:%d>" % (self.stream_id, self.weight, self.depends_on)
+		return "<h2frame_producer stream_id:%d, weight:%d, depends_on:%d>" % (self.stream_id, self.weight, self.depends_on)
 	
 	def is_done (self):
 		return self._end_stream and not self._buf
 	
 	def is_end_stream (self, data):
-		return len (data) < self.producer.buffer_size
+		return not data or len (data) < self.producer.buffer_size
 				 	
 	def more (self):
 		if self.is_done ():
 			return b''
 
 		if self._buf:
-			data, self._buf = self._buf [:self.BUFFER_SIZE], self._buf [self.BUFFER_SIZE:]
+			data, self._buf = self._buf [:self.SIZE_BUFFER], self._buf [self.SIZE_BUFFER:]
 				
 		else:
 			data = self.producer.more ()
 			self._end_stream = self.is_end_stream (data)
-			if len (data) > self.BUFFER_SIZE:
-				data, self._buf = data [:self.BUFFER_SIZE], data [self.BUFFER_SIZE:]
+			if len (data) > self.SIZE_BUFFER:
+				data, self._buf = data [:self.SIZE_BUFFER], data [self.SIZE_BUFFER:]
 		
 		# print ("MULTIPLEXING", self.stream_id, self.encoder.local_flow_control_window (self.stream_id))
 		with self._lock:
@@ -84,29 +92,29 @@ class h2data_producer:
 		return data_to_send
 	
 	
-class h2stream_producer (h2data_producer):
+class h2stream_producer (h2frame_producer):
 	def __repr__ (self):
-		return "<grpc_stream_producer stream_id:%d>" % (self.stream_id,)
-	
-	def close (self):
-		self._end_stream = True
+		return "<h2stream_producer stream_id:%d>" % (self.stream_id,)
 	
+	def get_size (self):			
+		return -1
+		
+	def is_end_stream (self, data):
+		return len (data) == 0
+		
 	def more (self):
 		if self.is_done ():
 			return b''
 
 		if self._buf:
-			data, self._buf = self._buf [:self.BUFFER_SIZE], self._buf [self.BUFFER_SIZE:]
+			data, self._buf = self._buf [:self.SIZE_BUFFER], self._buf [self.SIZE_BUFFER:]
 				
 		else:
 			data = self.producer.more ()
-			if not data:
-				self._end_stream = True
-				
-			if len (data) > self.BUFFER_SIZE:
-				data, self._buf = data [:self.BUFFER_SIZE], data [self.BUFFER_SIZE:]
+			self._end_stream = self.is_end_stream (data)
+			if len (data) > self.SIZE_BUFFER:
+				data, self._buf = data [:self.SIZE_BUFFER], data [self.SIZE_BUFFER:]
 		
-		# print ("MULTIPLEXING", self.stream_id, self.encoder.local_flow_control_window (self.stream_id))
 		with self._lock:
 			try:
 				self.encoder.send_data (
@@ -122,4 +130,11 @@ class h2stream_producer (h2data_producer):
 				return b''
 		
 		return data_to_send
-		
\ No newline at end of file
+
+class h2_globbing_producer (producers.globbing_producer):
+	def __init__ (self, stream_id, depends_on, weight,  producer, buffer_size = producers.SIZE_BUFFER):
+		self.stream_id = stream_id
+		self.depends_on = depends_on
+		self.weight = weight
+		producers.globbing_producer.__init__ (self, producer, buffer_size)
+		
