@@ -1,14 +1,15 @@
-from aquests.protocols.http import base_request_handler
+from ...protocols.http import base_request_handler
 from h2.connection import H2Connection, GoAwayFrame
-from h2.exceptions import ProtocolError, NoSuchStreamError, StreamClosedError
-from h2.events import DataReceived, ResponseReceived, StreamEnded, ConnectionTerminated, StreamReset, WindowUpdated
+from h2.exceptions import ProtocolError, NoSuchStreamError, StreamClosedError, FlowControlError
+from h2.events import DataReceived, ResponseReceived, StreamEnded, ConnectionTerminated, StreamReset, WindowUpdated, RemoteSettingsChanged
 from h2.errors import PROTOCOL_ERROR, FLOW_CONTROL_ERROR, NO_ERROR
 import h2.settings
-from aquests.lib import producers
-from .producers import h2data_producer, h2header_producer
-from aquests.protocols.http import respcodes
+from ...lib import producers
+from .producers import h2frame_producer, h2header_producer
+from ..http import respcodes
+from ..http import response as http_response
 
-from aquests.client import asynconnect
+from ...client import asynconnect
 import threading
 try:
 	from cStringIO import StringIO as BytesIO
@@ -17,9 +18,16 @@ except ImportError:
 
 class FakeAsynConnect:
 	collector = None
-	def __init__ (self, logger):
+	def __init__ (self, asyncon, logger):
+		self.asyncon = asyncon
 		self.logger = logger
 	
+	def get_proto (self):
+		return "h2"
+		
+	def is_proxy (self):
+		return False
+		
 	def handle_close (self, *args, **karg):
 		pass
 		
@@ -48,12 +56,13 @@ class FakeAsynConnect:
 		if self.collector:
 			self.collector.collect_incoming_data ()
 	
-	def collect_incoming_data (self):
-		if self.collector:
-			self.collector.collect_incoming_data ()
-		
+	def disconnect (self):
+		pass
+			
 	
 class RequestHandler (base_request_handler.RequestHandler):
+	# HTTP2 Adaptor Class	
+	# this class rely: asyncon - this class - http_reqyest_handler
 	def __init__ (self, handler):
 		self.asyncon = handler.asyncon
 		self.request = handler.request
@@ -61,9 +70,10 @@ class RequestHandler (base_request_handler.RequestHandler):
 		#self.asyncon.set_timeout (60, 60)
 		self.lock = handler.asyncon.lock # pool lock
 		self._ssl = handler._ssl
+		
 		self._clock = threading.RLock () # conn lock
 		self._llock = threading.RLock () # local lock
-		self.fakecon = FakeAsynConnect (self.logger)
+		self.fakecon = FakeAsynConnect (self.asyncon, self.logger)		
 		
 		self.initiate ()		
 		is_upgrade = not (self._ssl or self.request.initial_http_version == "2.0")			
@@ -72,7 +82,7 @@ class RequestHandler (base_request_handler.RequestHandler):
 			self.conn.update_settings({h2.settings.ENABLE_PUSH: 0})
 			# assume 1st request's stream_id = 1
 			self.add_request (1, handler)
-			self._send_stream_id = 1
+			self._send_stream_id = 1 # nexet request stream_id will be 3
 		else:		    
 			self.conn.initiate_connection()
 			
@@ -96,10 +106,13 @@ class RequestHandler (base_request_handler.RequestHandler):
 		self.frame_buf.max_frame_size = self.conn.max_inbound_frame_size		
 		self.data_length = 0
 		self.current_frame = None
-			
+	
 	def working (self):
+		return self.jobs () and True or False
+	
+	def jobs (self):
 		with self._llock:
-			return len (self.requests)	
+			return len (self.requests)
 		
 	def go_away (self, errcode = 0, msg = None):
 		with self._plock:
@@ -129,13 +142,11 @@ class RequestHandler (base_request_handler.RequestHandler):
 			self.asyncon.push (data_to_send)
 							
 	def handle_request (self, handler):
-		if not self.asyncon.connected:
-			self.initiate ()
-			self.asyncon.connect ()
-			
 		self.request = handler.request
-		stream_id = self.get_new_stream_id ()
-		self.add_request (stream_id, handler)
+		stream_id = self.get_new_stream_id ()		
+		self.add_request (stream_id, handler)		
+		self.asyncon.set_active (False)		
+		
 		headers, content_encoded = handler.get_request_header ("2.0", False)
 		payload = handler.get_request_payload ()
 		producer = None
@@ -146,33 +157,24 @@ class RequestHandler (base_request_handler.RequestHandler):
 				)
 			else:
 				# multipart, grpc_producer 
-				producer = producers.globbing_producer (payload)				
+				producer = producers.globbing_producer (payload)
 		
 		header = h2header_producer (stream_id, headers, producer, self.conn, self._clock)		
 		self.asyncon.push (header)				
 		if producer:
-			payload = h2data_producer (stream_id, 0, 1, producer, self.conn, self._clock)
+			payload = h2frame_producer (stream_id, 0, 1, producer, self.conn, self._clock)
 			self.asyncon.push (payload)
-		
-		if self.asyncon.connected:		
-			# IMP:  why?
-			rfcw = self.conn.remote_flow_control_window (stream_id)
-			if rfcw < 16384:
-				self.increment_flow_control_window (stream_id, 65535)
-				
-		self.asyncon.set_active (False)
-			
-	def increment_flow_control_window (self, stream_id, cl):
-		rfcw = self.conn.remote_flow_control_window (stream_id)
-		if cl > rfcw:
-			try:
-				self.conn.increment_flow_control_window (cl - rfcw, stream_id)
-			except StreamClosedError:
-				pass
-			else:
+	
+	def increment_flow_control_window (self, cl, stream_id = 0):
+		if not stream_id:
+			with self._clock:
 				self.conn.increment_flow_control_window (cl)
-							
-		self.send_data ()
+				self.send_data ()				
+		else:	
+			with self._clock:
+				try: self.conn.increment_flow_control_window (cl, stream_id)
+				except StreamClosedError: pass
+				else: self.send_data ()	
 					
 	def collect_incoming_data (self, data):
 		if not data:  return
@@ -182,13 +184,14 @@ class RequestHandler (base_request_handler.RequestHandler):
 			self.buf += data
 	
 	def connection_closed (self, why, msg):
-		with self._llock:			
+		with self._llock:
 			requests = list (self.requests.items ())
-		for stream_id, request in requests:
-			request.connection_closed (why, msg)
-		with self._llock:	
+		for stream_id, h in requests:
+			h.response = http_response.FailedResponse (720, respcodes.get (720), h.request)			
+			h.handle_callback ()
+		with self._llock:
 			self.requests = {}
-										
+
 	def found_terminator (self):
 		buf, self.buf = self.buf, b""		
 		events = None
@@ -249,36 +252,45 @@ class RequestHandler (base_request_handler.RequestHandler):
 			h.found_terminator ()
 		
 		# finally, make inactive on post, put request 
-		self.asyncon.set_active (False)	
+		self.asyncon.set_active (False)
 		
 	def handle_events (self, events):
 		for event in events:
 			#print (event)
 			if isinstance(event, ResponseReceived):
-				self.handle_response (event.stream_id, event.headers)		
+				self.handle_response (event.stream_id, event.headers)
 					
 			elif isinstance(event, StreamReset):
 				if event.remote_reset:				
 					h = self.get_handler (event.stream_id)
 					if h:
-						h.connection_closed (720, "HTTP2 Stream Rest")
+						h.response = http_response.FailedResponse (721, respcodes.get (721), h.request)
+						h.handle_callback ()
 						self.remove_handler (event.stream_id)
 					
 			elif isinstance(event, ConnectionTerminated):
 				self.asyncon.handle_close (721, "HTTP2 Connection Terminated")
 				
-			elif isinstance(event, DataReceived):
+			elif isinstance (event, DataReceived):
 				h = self.get_handler (event.stream_id)
 				if h:
 					h.collect_incoming_data (event.data)
 					rfcw = self.conn.remote_flow_control_window (event.stream_id)
-					if rfcw < 131070:
-						self.increment_flow_control_window (event.stream_id, 1048576)
+					if rfcw <= 16384:
+						self.increment_flow_control_window (1048576, event.stream_id)
+			
+			elif isinstance (event, RemoteSettingsChanged):
+				try:
+					iws = event.changed_settings [h2.settings.INITIAL_WINDOW_SIZE].new_value
+				except KeyError:
+					pass
+				else:		
+					self.increment_flow_control_window ((2 ** 31 - 1) - iws)
 				
-			elif isinstance(event, StreamEnded):				
+			elif isinstance(event, StreamEnded):
 				h = self.get_handler (event.stream_id)
 				if h:
-					#h.found_terminator ()
 					self.remove_handler (event.stream_id)
-					h.callback (h)
+					h.found_terminator ()					
 		self.send_data ()
+		
\ No newline at end of file
