@@ -14,8 +14,8 @@ def make_ws (_method, url, params, auth, headers, meta, proxy, logger):
 			handler_class = tunnel_handler.WSProxyTunnelHandler
 	else:		
 		handler_class = ws_request_handler.RequestHandler
-	return req, handler_class
-
+	req.handler = handler_class	
+	return req
 
 content_types = {
 	'xml': "text/xml",
@@ -47,17 +47,20 @@ def make_http (_method, url, params, auth, headers, meta, proxy, logger):
 			req = http_request.HTTPRequest (url, _method.upper (), params, headers, auth, logger, meta)
 			
 		else:	
-			if _method in ("post", "put"):
+			if _method in ("post", "put", "patch"):
 				ct = None
 				for k, v in headers.items ():
 					if k.lower () == "content-type":
 						ct = v						
 						break
-				if not ct: raise TypeError ("Content Type Undefined")
+				if not ct:
+					headers["Content-Type"] = "application/json"
 			
 			elif _method != "upload":
 				if _method [:4] == "post":					
-					cta, _method = _method [4:], "post"					 
+					cta, _method = _method [4:], "post"
+				elif _method [:4] == "patch":		
+					cta, _method = _method [5:], "patch"
 				else:
 					cta, _method = _method [3:], "put"							
 				ct = content_types.get (cta)
@@ -68,9 +71,9 @@ def make_http (_method, url, params, auth, headers, meta, proxy, logger):
 				req = http_request.HTTPMultipartRequest (url, "POST", params, headers, auth, logger, meta)
 			else:
 				req = http_request.HTTPRequest (url, _method.upper (), params, headers, auth, logger, meta)	
-			
-	return req, handler_class
-
-def make_dbo (_method, server, dbmethod, params, dbname, meta, logger):
-	return dbo_request.Request (_method [1:], server, dbname, dbmethod, params, None, meta)
 	
+	req.handler = handler_class
+	return req
+
+def make_dbo (_method, server, dbmethod, params, dbname, auth, meta, logger):
+	return dbo_request.Request (_method [1:], server, dbname, auth, dbmethod, params, None, meta)
